function [data,type] = IFDdata(fid, tag, IFDoffset);
  
% read raw data from image file directory
%
% fid ... file id (must be open)
% tag ... tag id to read, see TIFF and DNG specification
% IFDoffset ... optional, allows to scan EXIF data

fseek(fid, 0);
TIFFID = fread(fid, 4, 'uint8');
if TIFFID != [73 73 42 0]
  error('file is no TIFF or DNG file');
end

if nargin < 3
  IFDoffset = fread(fid, 1, 'uint32');
end
fseek(fid, IFDoffset);

while (IFDentries = fread(fid, 1, 'uint16')) != 0
  for i=1:IFDentries
    _tag = fread(fid, 1, 'uint16');
    if _tag == tag
      type = fread(fid, 1, 'uint16');
      numdata = fread(fid, 1, 'uint32');
      switch type
        case 1 % BYTE = uint8
          if numdata > 4
            offset = fread(fid, 1, 'uint32');
            fseek(fid, offset);
          end
          data = fread(fid, numdata, 'uint8')';
        case 2 % ASCII
          if numdata > 4
            offset = fread(fid, 1, 'uint32');
            fseek(fid, offset);
          end
          data = char(fread(fid, numdata, 'uchar')');
        case 3 % SHORT = uint16
          if numdata > 2
            offset = fread(fid, 1, 'uint32');
            fseek(fid, offset);
          end
          data = fread(fid, numdata, 'uint16')';
        case 4 % LONG = uint32
          if numdata > 1
            offset = fread(fid, 1, 'uint32');
            fseek(fid, offset);
          end
          data = fread(fid, numdata, 'uint32')';
        case 5 % RATIONAL = uint32/uint32
          offset = fread(fid, 1, 'uint32');
          fseek(fid, offset);
          data = fread(fid, 2*numdata, 'uint32')';
          data = data(1:2:end) ./ data(2:2:end);
        case 6 % SBYTE = int8
          if numdata > 4
            offset = fread(fid, 1, 'uint32');
            fseek(fid, offset);
          end
          data = fread(fid, numdata, 'int8')';
        case 7 % UNDEFINED 8 bit byte, used in ExifVersion as 4x char
          if numdata > 4
            offset = fread(fid, 1, 'uint32');
            fseek(fid, offset);
          end
          data = char(fread(fid, numdata, 'uchar')');
        case 8 % SSHORT = int16
          if numdata > 2
            offset = fread(fid, 1, 'uint32');
            fseek(fid, offset);
          end
          data = fread(fid, numdata, 'int16')';
        case 9 % SLONG = int32
          if numdata > 1
            offset = fread(fid, 1, 'uint32');
            fseek(fid, offset);
          end
          data = fread(fid, numdata, 'int32')';
        case 10 % SRATIONAL = int32 / int32
          data = fread(fid, 1, 'uint32');
          fseek(fid, data);
          data = fread(fid, 2*numdata, 'int32')';
          data = data(1:2:end) ./ data(2:2:end);
        case 11 % FLOAT (32 bit)
          if numdata > 1
            offset = fread(fid, 1, 'uint32');
            fseek(fid, offset);
          end
          data = fread(fid, numdata, 'float')';
        case 12 % DOUBLE (64 bit)
          offset = fread(fid, 1, 'uint32');
          fseek(fid, offset);
          data = fread(fid, numdata, 'float')';
      otherwise
        data = [];
        type 
      end
      return;
    end
    fread(fid, 10, 'uint8'); % dummy read, skip to next
  end
end
data = [];
type = 0;