function rgb = bayer2rgb (bayer)

% function rgb = bayer2rgb (bayer)
%
% extract r,g,b from bayer pattern, 
% combine to 3D rgb array
%
% NOTE: you can change the algorithm to use with the switch statement below
% 1 ... simple, fast but reduction of output image size by factor of 2
% 2 ... bilinear interpolation, slower but same size

switch 2 % select algorithm
  case 1 % combine each 2x2 to one rgb pixel = binning
    rgb = zeros(size(bayer,1)/2, size(bayer,2)/2, 3);
    % red = upper right
    rgb(:,:,1) = bayer(1:2:end, 2:2:end); 
    % green = average of upper left and lower right
    rgb(:,:,2) = (bayer(1:2:end, 1:2:end)+bayer(2:2:end, 2:2:end))/2; 
    % blue = lower left
    rgb(:,:,3) = bayer(2:2:end, 1:2:end); 
  case 2 % bilinear filtering
    rgb = zeros(size(bayer,1), size(bayer,2), 3);

    % interpolate red 
    r = bayer(1:2:end, 2:2:end);
    % within odd lines
    rgb(1:2:end, 2:2:end, 1) = r; % copy
    rgb(1:2:end, 1:2:end, 1) = [r(:,1) (r(:,1:end-1)+r(:,2:end))/2];
    %   even red lines
    rgb(2:2:end-1, :, 1) = (rgb(1:2:end-2,:,1) + rgb(3:2:end,:,1))/2;
    rgb(end, :, 1) = rgb(end-1, :, 1); % last line is copy of previous
    
    % interpolate green
    g1 = bayer(1:2:end, 1:2:end);
    g2 = bayer(2:2:end, 2:2:end);
    rgb(1:2:end, 1:2:end, 2) = g1; % copy
    rgb(2:2:end, 2:2:end, 2) = g2; % copy
    % pixels inside are average of 4 neighbors (top, bottom, left, right)
    rgb(2:2:end-2, 3:2:end, 2) = (g1(1:end-1, 2:end) + g1(2:end, 2:end) + g2(1:end-1, 1:end-1) + g2(1:end-1, 2:end))/4;
    rgb(3:2:end, 2:2:end-2, 2) = (g2(1:end-1, 1:end-1) + g2(2:end, 2:end) + g1(2:end, 1:end-1) + g1(2:end, 1:end-1))/4;
    % todo pixels along edges
    rgb(1, 2:2:end-1, 2) = (g1(1, 1:end-1)+g1(1, 2:end))/2; % top
    rgb(end, 3:2:end, 2) = (g2(end, 1:end-1)+g2(end, 2:end))/2; % bottom
    rgb(2:2:end-1, 1, 2) = (g1(1:end-1, 1)+g1(2:end, 1))/2; % left
    rgb(3:2:end, end, 2) = (g2(1:end-1,end)+g2(2:end,end))/2; % right
    rgb(1, end, 2) = (g1(1,end) + g2(1,end))/2; % top right
    rgb(end, 1, 2) = (g1(end,1) + g2(end,1))/2; % bottom left

    % interpolate blue
    b = bayer(2:2:end, 1:2:end);
    %   within even lines
    rgb(2:2:end, 1:2:end, 3) = b; % copy
    rgb(2:2:end, 2:2:end, 3) = [(b(:,1:end-1)+b(:,2:end))/2 b(:,end)];
    % between lines
    rgb(3:2:end, :, 3) = (rgb(2:2:end-2, :, 3) + rgb(4:2:end, :, 3))/2;
    rgb(1, :, 3) = rgb(2, :, 3); % first line is copy of second
end
