% decode DNG or TIFF RAW color or monochrome frame
% without color calibration and color matrix processing
clear
clf

% my sample images, select your own
switch 1
  case 1
    filename = 'D:/HighSpeedCam/Color/DNG 16Bit.dng'; 
    isColor = 1;
  case 2
    filename = 'D:/HighSpeedCam/Color/TIFF RAW.tiff'; 
    isColor = 1;
  case 11
    filename = 'D:/HighSpeedCam/Mono/DNG 16Bit.dng'; 
    isColor = 0;
  case 12
    filename = 'D:/HighSpeedCam/Mono/TIFF RAW.tiff'; 
    isColor = 0;
end

Gamma = 2.2; % set gamma correction value

% open input file
fid = fopen(filename, 'rb'); % optional add 'ieee-le'
if fid<0, error('file not found'); end

% get required information
hsize  = IFDdata(fid, 256) % hsize tag
vsize  = IFDdata(fid, 257) % vsize tag

% some more image information
WhiteLevel = IFDdata(fid, hex2dec('C61D'));
if length(WhiteLevel) == 0
  WhiteLevel = 65535; % TIFF does not have this TAG
end

if isColor
  % try to get color matrix, else set default
  ColorMatrix1 = IFDdata(fid, hex2dec('C621'));
  if length(ColorMatrix1) == 9
    ColorMatrix1 = reshape(ColorMatrix1,3,3)';
  end
  % try to get white calibration, else set default
  AsShotNeutral = IFDdata(fid, hex2dec('C628'));
  % equals LightSource EXIF TAG 9208.H
  % DNG shoule reads value 20 = "D55" 
  CalibrationIlluminant1 = IFDdata(fid, hex2dec('C65A'));
end

% get actual image data
offset = IFDdata(fid, 273); % strip offset(s) - we expect just one
fseek(fid, offset); % start of image data

% 16 bit data (with 12 bit of actual sensor data)
rawdata = fread(fid, hsize*vsize, 'uint16');
fclose(fid);

img = reshape(rawdata/WhiteLevel, hsize, vsize)'; % scale and make matrix
if isColor
  img = processColor(img, AsShotNeutral, ColorMatrix1, CalibrationIlluminant1);
end
img = img .^ (1/Gamma); % some very simple gamma correction
imshow(img, [0 1]);
