% decode a frame of raw 16 or 12  bit color recording
% without color calibration and color matrix processing
clear
clf

hsize = 1920; % horzontal size in pixels
vsize = 1080; % vertical size in pixels
framenum = 1; % frame within recording

% my sample images, select your own
switch 12
  case 1
    filename = 'D:/HighSpeedCam/Color/Raw 16Bit 2frames.raw';
    bits = 16;
    isColor = 1;
  case 2
    filename = 'D:/HighSpeedCam/Color/Raw 12Bit 2frames.raw';
    bits = 12;
    isColor = 1;
  case 11
    filename = 'D:/HighSpeedCam/Mono/Raw 16Bit 2frames.raw';
    bits = 16;
    isColor = 0;
  case 12,
    filename = 'D:/HighSpeedCam/Mono/Raw 12Bit 2frames.raw';
    bits = 12;
    isColor = 0;
end

Gamma = 2.2; % set gamma correction value

% open input file
fid = fopen(filename, 'rb'); % optional add 'ieee-le'
if fid<0, error('file not found'); end
% set file position to start of requested frame
% read one frame's data and convert to range [0 ... 4095];
switch bits,
  case 16
    % 16 bit values, left justified
    fseek(fid, hsize*vsize*2*framenum);
    rawdata = fread(fid, hsize*vsize, 'uint16')/16;
case 12
    % two 12 bit values are packed into 3 bytes
    fseek(fid, hsize*vsize*3/2*framenum);
    rawdata = fread(fid, hsize*vsize*3/2, 'uint8');
    rawdata = raw12to16(rawdata); % convert packed data to 16 bit values
end
fclose(fid);

img = reshape(rawdata/4095, hsize, vsize)'; % scale and make matrix
if isColor
  img = processColor(img);
end
img = img .^ (1/Gamma); % some very simple gamma correction
imshow(img, [0 1]);
